package de.hftstuttgart.dtabackend.utils;

import com.github.dockerjava.api.DockerClient;
import com.github.dockerjava.api.command.CreateContainerResponse;
import com.github.dockerjava.api.exception.DockerException;
import com.github.dockerjava.api.model.Bind;
import com.github.dockerjava.api.model.HostConfig;
import com.github.dockerjava.api.model.Mount;
import com.github.dockerjava.api.model.MountType;
import com.github.dockerjava.api.model.Volume;
import com.github.dockerjava.core.DefaultDockerClientConfig;
import com.github.dockerjava.core.DockerClientConfig;
import com.github.dockerjava.core.DockerClientImpl;
import com.github.dockerjava.zerodep.ZerodepDockerHttpClient;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.Arrays;
import java.util.stream.Collectors;

@Component
public class DockerUtil {
    private static final Logger LOG = LogManager.getLogger(DockerUtil.class);

    private DockerClient dockerClient;

    public DockerUtil(Environment env) {
        LOG.info("initializing Docker tools");
        LOG.debug("create docker client config");
        DockerClientConfig dockerClientConfig = DefaultDockerClientConfig.createDefaultConfigBuilder().build();

        LOG.debug("create docker http client");
        ZerodepDockerHttpClient httpClient = new ZerodepDockerHttpClient.Builder()
                .dockerHost(dockerClientConfig.getDockerHost())
                .sslConfig(dockerClientConfig.getSSLConfig())
        		.build();

        LOG.debug("create docker client");
        dockerClient = DockerClientImpl.getInstance(dockerClientConfig, httpClient);
    }

    public int runContainerWithBinds(String image, Bind... binds) throws InterruptedException, IOException
    {
        LOG.debug(String.format("pull image: %s", image));
        try {
            dockerClient.pullImageCmd(image)
                .start()
                .awaitCompletion()
                .close();
        } catch (DockerException e) {
            LOG.error(String.format(
                "pulling docker image %s failed with %s, trying with local image",
                image,
                e.getMessage()));
        }

        LOG.debug("creating container");
        CreateContainerResponse containerResponse;
        try {
            containerResponse = dockerClient.createContainerCmd("testcontainer")
                .withImage(image)
                .withHostConfig(
                    HostConfig.newHostConfig()
                        .withBinds(binds))
                .exec();
        } catch (DockerException e) {
            LOG.error(String.format(
                "Creating Docker Testrunner container failed with %s", e.getMessage()));
            throw e;
        }

        LOG.debug(String.format("container created: %s", containerResponse.getId()));

        LOG.debug(String.format("starting container %s", containerResponse.getId()));
        dockerClient.startContainerCmd(containerResponse.getId()).exec();

        LOG.debug(String.format("waiting for completion of container %s", containerResponse.getId()));
        int ret = dockerClient
            .waitContainerCmd(containerResponse.getId())
            .start()
            .awaitCompletion()
            .awaitStatusCode();
        LOG.debug(String.format("container completed with status %d", ret));

        LOG.debug(String.format("deleting container %s", containerResponse.getId()));
        dockerClient.removeContainerCmd(containerResponse.getId())
            .withRemoveVolumes(true)
            .exec();

        return ret;
    }

    public int runContainerWithVolumes(String image, Volume... volumes) throws InterruptedException, IOException {
    LOG.debug(String.format("Pulling image: %s", image));
    try {
        dockerClient.pullImageCmd(image)
            .start()
            .awaitCompletion()
            .close();
    } catch (DockerException e) {
        LOG.error(String.format(
            "Pulling Docker image %s failed with %s, trying with local image",
            image,
            e.getMessage()));
    }

    LOG.debug("Creating container");
    CreateContainerResponse containerResponse;
    try {
        // Prepare the host configuration with volumes
        HostConfig hostConfig = HostConfig.newHostConfig()
            .withMounts(
                Arrays.stream(volumes)
                    .map(volume -> new Mount().withTarget(volume.getPath()).withType(MountType.VOLUME))
                    .collect(Collectors.toList())
            );

        // Create the container with the configured volumes
        containerResponse = dockerClient.createContainerCmd("testcontainer")
            .withImage(image)
            .withHostConfig(hostConfig)
            .exec();
    } catch (DockerException e) {
        LOG.error(String.format(
            "Creating Docker Testrunner container failed with %s", e.getMessage()));
        throw e;
    }

    LOG.debug(String.format("Container created: %s", containerResponse.getId()));

    LOG.debug(String.format("Starting container %s", containerResponse.getId()));
    dockerClient.startContainerCmd(containerResponse.getId()).exec();

    LOG.debug(String.format("Waiting for completion of container %s", containerResponse.getId()));
    int ret = dockerClient
        .waitContainerCmd(containerResponse.getId())
        .start()
        .awaitCompletion()
        .awaitStatusCode();
    LOG.debug(String.format("Container completed with status %d", ret));

    LOG.debug(String.format("Deleting container %s", containerResponse.getId()));
    dockerClient.removeContainerCmd(containerResponse.getId())
        .withRemoveVolumes(true)
        .exec();

    return ret;
}

}
